@@ -1,18 +1,19 @@
 # -*- coding: utf-8 -*-
 
 from decimal import Decimal
+from typing import Union
 
 from PyQt5.QtCore import pyqtSignal, Qt
-from PyQt5.QtGui import QPalette, QPainter, QFontMetrics
+from PyQt5.QtGui import QPalette, QPainter
 from PyQt5.QtWidgets import (QLineEdit, QStyle, QStyleOptionFrame)
 
-from .util import char_width_in_lineedit
+from .util import char_width_in_lineedit, ColorScheme
 
 from electrum.util import (format_satoshis_plain, decimal_point_to_base_unit_name,
                            FEERATE_PRECISION, quantize_feerate)
 
 
-class MyLineEdit(QLineEdit):
+class FreezableLineEdit(QLineEdit):
     frozen = pyqtSignal()
 
     def setFrozen(self, b):
@@ -20,7 +21,7 @@ class MyLineEdit(QLineEdit):
         self.setFrame(not b)
         self.frozen.emit()
 
-class AmountEdit(MyLineEdit):
+class AmountEdit(FreezableLineEdit):
     shortcut = pyqtSignal()
 
     def __init__(self, base_unit, is_int=False, parent=None):
@@ -31,7 +32,6 @@ class AmountEdit(MyLineEdit):
         self.textChanged.connect(self.numbify)
         self.is_int = is_int
         self.is_shortcut = False
-        self.help_palette = QPalette()
         self.extra_precision = 0
 
     def decimal_point(self):
@@ -68,10 +68,10 @@ class AmountEdit(MyLineEdit):
             textRect = self.style().subElementRect(QStyle.SE_LineEditContents, panel, self)
             textRect.adjust(2, 0, -10, 0)
             painter = QPainter(self)
-            painter.setPen(self.help_palette.brush(QPalette.Disabled, QPalette.Text).color())
-            painter.drawText(textRect, Qt.AlignRight | Qt.AlignVCenter, self.base_unit())
+            painter.setPen(ColorScheme.GRAY.as_color())
+            painter.drawText(textRect, int(Qt.AlignRight | Qt.AlignVCenter), self.base_unit())
 
-    def get_amount(self):
+    def get_amount(self) -> Union[None, Decimal, int]:
         try:
             return (int if self.is_int else Decimal)(str(self.text()))
         except:
@@ -91,6 +91,7 @@ class BTCAmountEdit(AmountEdit):
         return decimal_point_to_base_unit_name(self.decimal_point())
 
     def get_amount(self):
+        # returns amt in satoshis
         try:
             x = Decimal(str(self.text()))
         except:
@@ -105,11 +106,12 @@ class BTCAmountEdit(AmountEdit):
         amount = Decimal(max_prec_amount) / pow(10, self.max_precision()-self.decimal_point())
         return Decimal(amount) if not self.is_int else int(amount)
 
-    def setAmount(self, amount):
-        if amount is None:
-            self.setText(" ") # Space forces repaint in case units changed
+    def setAmount(self, amount_sat):
+        if amount_sat is None:
+            self.setText(" ")  # Space forces repaint in case units changed
         else:
-            self.setText(format_satoshis_plain(amount, self.decimal_point()))
+            self.setText(format_satoshis_plain(amount_sat, decimal_point=self.decimal_point()))
+        self.repaint()  # macOS hack for #6269
 
 
 class FeerateEdit(BTCAmountEdit):
