@@ -9,8 +9,9 @@ from PyQt5.QtWidgets import (
 )
 
 from electrum.i18n import _
+from electrum.simple_config import SimpleConfig
 
-from .util import WindowModalDialog, get_parent_main_window, WWLabel
+from .util import WindowModalDialog, WWLabel, getSaveFileName
 
 
 class QRCodeWidget(QWidget):
@@ -71,7 +72,7 @@ class QRCodeWidget(QWidget):
 
         margin = 10
         framesize = min(r.width(), r.height())
-        boxsize = int( (framesize - 2*margin)/k )
+        boxsize = int((framesize - 2*margin)/k)
         size = k*boxsize
         left = (framesize - size)/2
         top = (framesize - size)/2
@@ -95,19 +96,26 @@ class QRDialog(WindowModalDialog):
 
     def __init__(
             self,
+            *,
             data,
             parent=None,
             title="",
             show_text=False,
-            *,
             help_text=None,
             show_copy_text_btn=False,
+            config: SimpleConfig,
     ):
         WindowModalDialog.__init__(self, parent, title)
+        self.config = config
 
         vbox = QVBoxLayout()
+
         qrw = QRCodeWidget(data)
-        vbox.addWidget(qrw, 1)
+        qr_hbox = QHBoxLayout()
+        qr_hbox.addWidget(qrw)
+        qr_hbox.addStretch(1)
+        vbox.addLayout(qr_hbox)
+
         help_text = data if show_text else help_text
         if help_text:
             text_label = WWLabel()
@@ -117,14 +125,15 @@ class QRDialog(WindowModalDialog):
         hbox.addStretch(1)
 
         def print_qr():
-            main_window = get_parent_main_window(self)
-            if main_window:
-                filename = main_window.getSaveFileName(_("Select where to save file"), "qrcode.png")
-            else:
-                filename, __ = QFileDialog.getSaveFileName(self, _("Select where to save file"), "qrcode.png")
+            filename = getSaveFileName(
+                parent=self,
+                title=_("Select where to save file"),
+                filename="qrcode.png",
+                config=self.config,
+            )
             if not filename:
                 return
-            p = qrw.grab()  # FIXME also grabs neutral colored padding
+            p = qrw.grab()
             p.save(filename, 'png')
             self.show_message(_("QR code saved to file") + " " + filename)
 
