@@ -25,6 +25,9 @@ import electrum.ecc as ecc
 from ..hw_wallet import HW_PluginBase, HardwareClientBase
 
 
+_logger = get_logger(__name__)
+
+
 try:
     from bitbox02 import bitbox02
     from bitbox02 import util
@@ -36,13 +39,12 @@ try:
         FirmwareVersionOutdatedException,
     )
     requirements_ok = True
-except ImportError:
+except ImportError as e:
+    if not (isinstance(e, ModuleNotFoundError) and e.name == 'bitbox02'):
+        _logger.exception('error importing bitbox02 plugin deps')
     requirements_ok = False
 
 
-_logger = get_logger(__name__)
-
-
 class BitBox02Client(HardwareClientBase):
     # handler is a BitBox02_Handler, importing it would lead to a circular dependency
     def __init__(self, handler: Any, device: Device, config: SimpleConfig, *, plugin: HW_PluginBase):
@@ -293,8 +295,7 @@ class BitBox02Client(HardwareClientBase):
             raise Exception(
                 "Need to setup communication first before attempting any BitBox02 calls"
             )
-
-        account_keypath = bip32_path[:4]
+        account_keypath = bip32_path[:-2]
         xpubs = wallet.get_master_public_keys()
         our_xpub = self.get_xpub(
             bip32.convert_bip32_intpath_to_strpath(account_keypath), xtype
@@ -504,15 +505,7 @@ class BitBox02Client(HardwareClientBase):
                     )
                 )
 
-        if type(wallet) is Standard_Wallet:
-            keypath_account = full_path[:3]
-        elif type(wallet) is Multisig_Wallet:
-            keypath_account = full_path[:4]
-        else:
-            raise Exception(
-                "BitBox02 does not support this wallet type: {}".format(type(wallet))
-            )
-
+        keypath_account = full_path[:-2]
         sigs = self.bitbox02_device.btc_sign(
             coin,
             [bitbox02.btc.BTCScriptConfigWithKeypath(
@@ -636,7 +629,7 @@ class BitBox02_KeyStore(Hardware_KeyStore):
 
 class BitBox02Plugin(HW_PluginBase):
     keystore_class = BitBox02_KeyStore
-    minimum_library = (5, 0, 0)
+    minimum_library = (5, 2, 0)
     DEVICE_IDS = [(0x03EB, 0x2403)]
 
     SUPPORTED_XTYPES = ("p2wpkh-p2sh", "p2wpkh", "p2wsh", "p2wsh-p2sh")
